﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftCarrier
{
    internal class DrawingAircraftCarrier : DrawingWarship
    {
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес военного корабля</param>
        /// <param name="bodyColor">Цвет основной палубы</param>
        /// <param name="dopColor">Дополнительный цвет</param>
        /// <param name="bodyKit">Признак наличия боковой площадки</param>
        /// <param name="сabin">Признак наличия рубки</param>
        /// <param name="superEngine">Признак наличия усиленного двигателя</param>
        public DrawingAircraftCarrier(int speed, float weight, Color bodyColor, Color dopColor, bool bodyKit, bool сabin, bool superEngine) :
            base(speed, weight, bodyColor, 114, 40)
        {
            Warship = new EntityAircraftCarrier(speed, weight, bodyColor, dopColor, bodyKit, сabin, superEngine);
        }
        public void SetDopColor(Color color)
        {
            ((EntityAircraftCarrier)Warship).DopColor = color;
        }

        public override void DrawTransport(Graphics g)
        {
            if(Warship is not EntityAircraftCarrier aircraftCarrier)
            {
                return;
            }

            Pen pen = new(Color.Black);
            Brush dopBrush = new SolidBrush(aircraftCarrier.DopColor);
            Brush br = new SolidBrush(Warship?.BodyColor ?? Color.White);

            if (aircraftCarrier.BodyKit)
            {
               //боковая площадка
                PointF pointAircraftCarrier1 = new PointF(_startPosX + 94, _startPosY + 40);
                PointF pointAircraftCarrier2 = new PointF(_startPosX + 74, _startPosY + 60);
                PointF pointAircraftCarrier3 = new PointF(_startPosX + 24, _startPosY + 60);
                PointF pointAircraftCarrier4 = new PointF(_startPosX + 4, _startPosY + 40);

                PointF[] PointsAircraftCarrier =
                {
                    pointAircraftCarrier1, pointAircraftCarrier2, pointAircraftCarrier3, pointAircraftCarrier4
                };
                g.FillPolygon(br, PointsAircraftCarrier);
                g.DrawPolygon(pen, PointsAircraftCarrier);

                //полоса
                PointF pontStartLine1 = new PointF(_startPosX + 4, _startPosY);
                PointF pontStartLine2 = new PointF(_startPosX + 15, _startPosY);
                PointF pontStartLine3 = new PointF(_startPosX + 74, _startPosY + 60);
                PointF pontStartLine4 = new PointF(_startPosX + 59, _startPosY + 60);

                PointF[] PointsStartLine =
                {
                    pontStartLine1, pontStartLine2, pontStartLine3, pontStartLine4
                };
                g.FillPolygon(dopBrush, PointsStartLine);
                g.DrawPolygon(pen, PointsStartLine);
            }

            if (aircraftCarrier.SuperEngine)
            {
                g.FillEllipse(dopBrush, _startPosX, _startPosY, 10, 10);
                g.DrawEllipse(pen, _startPosX, _startPosY, 10, 10);

                g.FillEllipse(dopBrush, _startPosX, _startPosY + 10, 10, 10);
                g.DrawEllipse(pen, _startPosX, _startPosY + 10, 10, 10);

                g.FillEllipse(dopBrush, _startPosX, _startPosY + 18, 10, 10);
                g.DrawEllipse(pen, _startPosX, _startPosY + 18, 10, 10);

                g.FillEllipse(dopBrush, _startPosX, _startPosY + 30, 10, 10);
                g.DrawEllipse(pen, _startPosX, _startPosY + 30, 10, 10);
            }

            base.DrawTransport(g);          

            if (aircraftCarrier.Сabin)
            {
                g.FillEllipse(dopBrush, _startPosX + 10, _startPosY + 15, 10, 10);
                g.DrawEllipse(pen, _startPosX + 10, _startPosY + 15, 10, 10);                

                g.FillEllipse(dopBrush, _startPosX + 20, _startPosY + 15, 10, 10);
                g.DrawEllipse(pen, _startPosX + 20, _startPosY + 15, 10, 10);

                g.FillRectangle(dopBrush, _startPosX + 15, _startPosY + 15, 10, 10);
                g.DrawRectangle(pen, _startPosX + 15, _startPosY + 15, 10, 10);
            }
        }
    }

}
