﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftCarrier
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetWarshipsGeneric<T>
        where T : class
    {
        /// <summary>
		/// Список объектов, которые храним
		/// </summary>
		private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;

        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetWarshipsGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="warship">Добавляемый военный корабль</param>
        /// <returns></returns>
        public int Insert(T warship)
        {
            return Insert(warship, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="warship">Добавляемый военный корабль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T warship, int position)
        {
            if (position >= _maxCount || position < 0) return -1;
            _places.Insert(position, warship);
            return 1;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position >= _maxCount || position < 0)
                return null;

            var result = _places[position];
            _places.RemoveAt(position);
            return result;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position >= _maxCount || position < 0) return null;
                return _places[position];
            }
            set
            {
                Insert(value, position);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetWarships()
        {
            foreach(var warship in _places)
            {
                if(warship != null)
                {
                    yield return warship;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
